#This script has been constructed for educational purposes at the
#University of Ioannina, Computer Science Department

#============Simulation Parameters (not to be changed)==============
set val(chan)           Channel/Sat	;#Channel Type
set val(netif)          Phy/Sat		;# network interface type
set val(mac)            Mac/Sat		;# MAC type
set val(ifq)            Queue/DropTail	;# interface queue type
set val(ll)             LL/Sat/HDLC	;# link layer type
set val(seed)		0
set val(err)            MarkovErrorModel
set val(stop)		10
set val(stopcbr) 	10
#=====Simulation Parameters (may be configured - default values)=====
# set selective repeat on; its GoBackN by default
set val(selective_repeat)	0
#set the sliding window width
set val(window_size)		1
#set the timeout time to wait for an acknowledgement
set val(timeout)		0.6
#set the error_rate. This is the probability of error happening
#in a packet, byte or bit depending the definition.
#In our example it is the propability of error in a packet
set val(error_rate)		0
#set the transmition speed (in bps)
set val(transmition_speed)	10000000
#set the packet size in bytes (keep in mind that LL will add 28 bytes for header
set val(packet_size)		1000
#Choose terminal location
set val(location)		1
#====================================================================

#==========================Procedures================================
proc getopt {argc argv} {
	global opt
	lappend optlist sr wsize er tr_speed location

	set count_input 0
	
	for {set i 0} {$i < $argc} {incr i} {
		set arg [lindex $argv $i]
		if {[string range $arg 0 0] != "-"} continue

		set name [string range $arg 1 end]
		
		if { $name == "sr" || $name == "wsize" || $name == "er" || $name == "tr_speed" || $name == "location" } { 
			set count_input [expr $count_input + 1]	
		}
		
		set opt($name) [lindex $argv [expr $i+1]]
	}
	
	if {$count_input != 5} {
		usage "Wrong number of arguments"
		exit 1
	}
}

proc usage { argv0 }  {
	puts "Usage: $argv0"
	puts "\tmandatory arguments:"
	puts "\t\t\[-sr 0 (Go-Back-N) or 1 (Selective Repeat)\] \[-wsize value of the window size\]"
	puts "\t\t\[-er value of error rate (0,1)\] \[-tr_speed value of transmission speed in bps\]"
	puts "\t\t\[-location 1 or 2 or 3 for selecting the terminal location\]\n"
}

proc finish {} {

	global ns_ val tracefd

	$ns_ flush-trace
	close $tracefd
	exit 0
}

proc compute_interval {} {

	#Do not change this computation. It is configured to provide
	# a constant flow of packets
	global val
	set temp			[expr (($val(transmition_speed)/(8*$val(packet_size))) + 500)]
	if {$temp < 10} {
		set temp1		[expr 10/$temp]
		set temp2		0.$temp1
	} else {
		if {$temp < 100} {
		set temp1		[expr 100/$temp]
		set temp2		0.0$temp1
		} else {
			if {$temp < 1000} {
				set temp1		[expr 1000/$temp]
				set temp2		0.00$temp1
			} else {
				if {$temp < 10000} {
					set temp1		[expr 10000/$temp]
					set temp2		0.000$temp1
				} else {
					if {$temp < 100000} {
						set temp1		[expr 100000/$temp]
						set temp2		0.0000$temp1
					}
				}
			}
		}
	}
	return $temp2
	exit 0
}
#====================================================================

#==========MAIN CODE=================================================
getopt $argc $argv

if { $opt(sr) != 0 && $opt(sr) != 1} {
	usage $argv0
	exit 1
}

if { $opt(wsize) == 0 } {
	usage $argv0
	exit 1
}

if { $opt(er) < 0 || $opt(er) > 1 } {
	usage $argv0
	exit 1
}

if { $opt(tr_speed) == 0 } {
	usage $argv0
	exit 1
}

if { $opt(location) != 1 && $opt(location) != 2 && $opt(location) != 3 } {
	usage $argv0
	exit 1
}

set val(selective_repeat) $opt(sr)
set val(window_size) $opt(wsize)
set val(error_rate) $opt(er)
set val(transmition_speed) $opt(tr_speed)
set val(location) $opt(location)


set val(bw_up)          $val(transmition_speed)
set val(bw_down)        $val(transmition_speed)
set val(ifqlen) 	[expr 3*$val(window_size) + 10]		;# intefrace queue size
LL/Sat/HDLC set window_size_ $val(window_size)
LL/Sat/HDLC set timeout_ $val(timeout)
LL/Sat/HDLC set max_timeouts_ 1000
LL/Sat/HDLC set selRepeat_ $val(selective_repeat)

set ns_	[new Simulator]
ns-random $val(seed)
set val(interval)	[compute_interval]

set tracefd     [open lab7.tr w]
$ns_ trace-all $tracefd
#$ns_ eventtrace-all

$ns_ rtproto Static
	
# Create sat n(0) configure node, please note the change below.

$ns_ node-config -satNodeType geo \
    -llType $val(ll) \
    -ifqType $val(ifq) \
    -ifqLen $val(ifqlen) \
    -macType $val(mac) \
    -phyType $val(netif) \
    -channelType $val(chan) \
    -downlinkBW $val(bw_down) \
    -wiredRouting ON

set n(0) [$ns_ node]
$n(0) set-position -95

$ns_ node-config -satNodeType terminal

set n(100) [$ns_ node]
if { $val(location) == 1 } {
	$n(100) set-position 37.9 -122.3; # Position1: Propagation delay ~126.7 msecs
}
if { $val(location) == 2 } {
	$n(100) set-position 39.67 20.82; # Position2: Propagation delay ~149.2 msecs
} 
if { $val(location) == 3 } {
	$n(100) set-position 38.22 -35.30; # Position3: Propagation delay ~133.6 msecs
}

# Add GSLs to geo satellites
$n(100) add-gsl geo $val(ll) $val(ifq) $val(ifqlen) $val(mac) $val(bw_up) \
    $val(netif) [$n(0) set downlink_] [$n(0) set uplink_]
	
# Add an error model to the sending dirction of geo node
set em_ [new ErrorModel]
$em_ unit pkt
$em_ set rate_ $val(error_rate)
$em_ ranvar [new RandomVariable/Uniform]

#Errors on the forward link
$n(0) interface-errormodel $em_ 

$ns_ trace-all-satlinks $tracefd

set satrouteobject_ [new SatRouteObject]
$satrouteobject_ compute_routes

set udp100 [new Agent/UDP]
set null0 [new Agent/Null]

$ns_ attach-agent $n(100) $udp100
$ns_ attach-agent $n(0) $null0

set cbr100 [new Application/Traffic/CBR]
$cbr100 set packetSize_ $val(packet_size)
$cbr100 set interval_ $val(interval)
$cbr100 attach-agent $udp100

$ns_ connect $udp100 $null0

$ns_ at 0 "$cbr100 start"

$ns_ at $val(stopcbr) "$cbr100 stop"

$ns_ at $val(stop) "finish"
$ns_ run
#================END OF MAIN=========================================






